%% Generates the figures used in "Segmentation of ARX-models Using
%% Sum-of-Norms Regularization
clear all
close all

%% Example 1, arx, changing time delay
load iddemo6m.mat
Y = z(3:end,1);
Phi = [-z(2:end-1,1)  z(2:end-1,2)  z(1:end-2,2)];
zi=iddata(z(:,1),z(:,2));
[theta v] = our_method(Phi,Y,0.1); % uses CVX

% figure 1, set line width and color manually
figure
plot(zi)
subplot(211)
title('')
ylabel('y')
xlabel('t')
subplot(212)
title('')
ylabel('u')
xlabel('t')
%print -dpsc2 'ex1data.eps'

% figure 2
figure
subplot(311)
plot(4:41,-squeeze(theta(end,1,:))','k','linewidth',2)
hold on
plot(-pars(:,1),'k--','linewidth',2)
xlabel('t')
ylabel('a')
axis([3 40 -0.25 1.25])
subplot(312)
plot(4:41,squeeze(theta(end,2,:))','k','linewidth',2)
hold on
plot(pars(:,2),'k--','linewidth',2)
xlabel('t')
ylabel('b_1')
axis([3 40 -0.25 1.25])
subplot(313)
plot(4:41,squeeze(theta(end,3,:))','k','linewidth',2)
hold on
plot(pars(:,3),'k--','linewidth',2)
xlabel('t')
ylabel('b_2')
axis([3 40 -0.25 1.25])
%print -dpsc2 'ex1.eps'

%% Example 2, changing coeff.
y=zeros(2,1);
A=1;B=1;
u=randn(200,1);
for k=3:200
if k<100, a = -1.5;else a=-1.3;end
y(k)=-a*y(k-1)-0.7*y(k-2)+u(k);
end
Y = y(3:end);
Phi = [y(2:end-1) y(1:end-2) u(3:end)];
[theta v] = our_method(Phi,Y,0.01);

% figure 3
figure
subplot(211)
plot(y,'k','linewidth',2)
xlabel('t')
ylabel('y')
axis([0 200 -7 7])
subplot(212)
plot(3:200,-squeeze(theta(end,1,:)),'k','linewidth',2)
xlabel('t')
ylabel('a')
axis([0 200 -1.52 -1.28])
%print -dpsc2 'ex2.eps'

% figure 4
figure
subplot(211)
plot(3:200,-squeeze(theta(1,1,:)),'linewidth',3,'color',[0.6 0.6 0.6])
hold on
plot(3:200,-squeeze(theta(2,1,:)),'color',[0.4 0.4 0.4],'linewidth',2)
plot(3:200,-squeeze(theta(3,1,:)),'color',[0.4 0.4 0.4],'linewidth',2)
plot(3:200,-squeeze(theta(4,1,:)),'k','linewidth',3)
axis([98 102 -1.51 -1.29])

subplot(212)
[N n] = size(Phi);
ratio_of_lambda_max = 0.01;
lambda_max = comp_lambdamax(Phi,Y)

% refinement #2
w = v(1,:);
w(v(1,:)<0.0005)=100000;
w(v(1,:)>0.0005)=0;
[theta2(2,:,:) v(2,:)] = segreg_sumofnorms(Phi,Y,lambda_max*ratio_of_lambda_max,w);

plot(3:200,-squeeze(theta2(2,1,:)),'k','linewidth',3)
hold on
plot(3:200,-squeeze(theta(1,1,:)),'color',[0.6 .6 0.6],'linewidth',3)
axis([98 102 -1.51 -1.29])
%print -dpsc2 'ex2mot.eps'

%% Example 3, earth quake
load quake
y = decimate(e,100);
Y = y(3:end);
Phi = [y(2:end-1) y(1:end-2)];
[theta v] = our_method(Phi,Y,0.15);

% figure 5
figure
subplot(211)
plot(y,'k','linewidth',2)
axis([0 100 -300 300])
ylabel('y')
xlabel('t')
subplot(212)
plot(squeeze(theta(end,1,:))','k','linewidth',2)
axis([0 100 -1.4 -0.3])
ylabel('a_1')
xlabel('t')
%print -dpsc2 'ex3.eps'

%% Example 4, hodata
u = randn(2000,1);
e= randn(2000,1);
y= zeros(2000,1);
for k=3:2000
    if k< 400
        a1 = -1.5;
        a2 = 0.7;
        b1=1;
        b2=0.5;
    elseif k<1500
        a1= -1.3;
        a2= 0.7;
        b1=1;
        b2=0.5;
    else
        a1=-1.5;
        a2=0.7;
        b1 = 1;
        b2= 0.5;
    end
    y(k) = -a1*y(k-1) - a2*y(k-2)+b1*u(k-1) + b2*u(k-2)+3*e(k);
end
z = iddata(y,u);
nn=[2 2 1];
[N,ny,nu]=size(z);
na = nn(1);
nb = nn(2:1+nu);
nk = nn(nu+2:2*nu+1);
yy = z.y; uu = z.u;
nm = max([na,nb+nk-1]);
y = yy(nm+1:end);
for k=1:N-nm
    C{k}=[-yy(nm+k-1:-1:nm+k-na)'];
    for ku=1:nu
        C{k}=[C{k},uu(nm+k-nk(ku):-1:nm+k-nb(ku)-nk(ku)+1,ku)'];
    end
end
Phi = cell2mat(C');
[theta v] = our_method(Phi,y,0.025);
[segm V] = segment(z,[2 2 1],5,0.001);
s1 = segment(z,[2 2 1],9,0.001,diag([sqrt(0.2),0,0,0]),10,[-1.5,0.7,1,0.5],zeros(4,4))

% figure 6
figure
subplot(2,1,1)
plot(3:2000,squeeze(theta(end,1,:))','k','linewidth',2)
hold on
plot(1:2000,segm(:,1),'--','color',[0.4 0.4 0.4],'linewidth',2)
%print -dpsc2 'ex4a.eps'
